import { Component, ElementRef, EventEmitter, Injectable, Input, NgModule, NgZone, Optional, Output, ViewChild, animate, forwardRef, style, transition, trigger } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, NG_VALUE_ACCESSOR, ReactiveFormsModule } from '@angular/forms';
import moment from 'moment';

var AngularDatepickerOptions = (function () {
    function AngularDatepickerOptions() {
        this.format = 'DD.MM.YYYY';
        this.viewFormat = 'DD.MM.YYYY';
        this.firstWeekdaySunday = false;
        this.todayButtonEnabled = true;
        this.todayHighlight = true;
        this.weekendHighlight = true;
        this.closeOnSelect = true;
    }
    return AngularDatepickerOptions;
}());
AngularDatepickerOptions.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
AngularDatepickerOptions.ctorParameters = function () { return []; };

var DatePickerComponent = (function () {
    /**
     * @param {?} elRef
     * @param {?} opts
     */
    function DatePickerComponent(elRef, opts) {
        var _this = this;
        this.elRef = elRef;
        this.opts = opts;
        this.options = {};
        this.date = moment();
        this.viewDate = null;
        this.days = [];
        this.onTouchedCallback = function () { };
        this.onChangeCallback = function () { };
        this.closePickerOnOutsideClick = function (event) { return _this.close(event); };
        this.closePickerOnTab = function (event) { return _this.closeOnTab(event); };
    }
    Object.defineProperty(DatePickerComponent.prototype, "value", {
        /**
         * @return {?}
         */
        get: function () {
            return this.viewDate;
        },
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            var /** @type {?} */ date = (value instanceof moment) ? value : moment(value, this.options.format);
            this.viewDate = date.isValid() ? date.format(this.options.viewFormat) : value;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} date
     * @return {?}
     */
    DatePickerComponent.prototype.onDateSelected = function (date) {
        this.value = date;
        this.onChangeCallback(date.format(this.options.format));
        if (this.options.closeOnSelect) {
            this.close();
        }
    };
    /**
     * @return {?}
     */
    DatePickerComponent.prototype.ngOnInit = function () {
        if (this.opts) {
            if (typeof this.options.format === 'undefined') {
                this.options.format = this.opts.format;
            }
            if (typeof this.options.viewFormat === 'undefined') {
                this.options.viewFormat = this.opts.viewFormat;
            }
            if (typeof this.options.modalMediaQuery === 'undefined') {
                this.options.modalMediaQuery = this.opts.modalMediaQuery;
            }
            if (typeof this.options.firstWeekdaySunday === 'undefined') {
                this.options.firstWeekdaySunday = this.opts.firstWeekdaySunday;
            }
            else {
                this.options.firstWeekdaySunday = this.options.firstWeekdaySunday.toString() === 'true';
            }
            if (typeof this.options.todayString === 'undefined') {
                this.options.todayString = this.opts.todayString;
            }
            if (typeof this.options.todayButtonEnabled === 'undefined') {
                this.options.todayButtonEnabled = this.opts.todayButtonEnabled;
            }
            else {
                this.options.todayButtonEnabled = this.options.todayButtonEnabled.toString() === 'true';
            }
            if (typeof this.options.todayHighlight === 'undefined') {
                this.options.todayHighlight = this.opts.todayHighlight;
            }
            else {
                this.options.todayHighlight = this.options.todayHighlight.toString() === 'true';
            }
            if (typeof this.options.weekendHighlight === 'undefined') {
                this.options.weekendHighlight = this.opts.weekendHighlight;
            }
            else {
                this.options.weekendHighlight = this.options.weekendHighlight.toString() === 'true';
            }
            if (typeof this.options.closeOnSelect === 'undefined') {
                this.options.closeOnSelect = this.opts.closeOnSelect;
            }
            else {
                this.options.closeOnSelect = this.options.closeOnSelect.toString() === 'true';
            }
        }
        this.datePicker.mergeOptions(this.options);
    };
    /**
     * @return {?}
     */
    DatePickerComponent.prototype.ngAfterViewInit = function () {
        if (this.opened) {
            this.datePicker.open();
        }
    };
    /**
     * @param {?} value
     * @return {?}
     */
    DatePickerComponent.prototype.writeValue = function (value) {
        this.viewDate = value;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    DatePickerComponent.prototype.registerOnChange = function (fn) {
        this.onChangeCallback = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    DatePickerComponent.prototype.registerOnTouched = function (fn) {
        this.onTouchedCallback = fn;
    };
    /**
     * @return {?}
     */
    DatePickerComponent.prototype.toggle = function () {
        if (!this.datePicker.opened) {
            this.open();
        }
        else {
            // this.close();
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    DatePickerComponent.prototype.closeOnTab = function (event) {
        if (event.keyCode === 9) {
            if (this.viewDate) {
                this.value = moment(this.viewDate, this.options.viewFormat);
            }
            this.onTouchedCallback();
            this.close();
        }
    };
    /**
     * @param {?=} event
     * @return {?}
     */
    DatePickerComponent.prototype.open = function (event) {
        if (event) {
            event.preventDefault();
        }
        var /** @type {?} */ date = moment(this.value, this.options.viewFormat);
        this.datePicker.inputDate = date.isValid() ? date : null;
        this.datePicker.open();
        window.addEventListener('click', this.closePickerOnOutsideClick, true);
        window.addEventListener('keydown', this.closePickerOnTab, true);
    };
    /**
     * @param {?=} event
     * @return {?}
     */
    DatePickerComponent.prototype.close = function (event) {
        if (event && this.elRef.nativeElement.contains(event.target)) {
            return;
        }
        this.onTouchedCallback();
        window.removeEventListener('keydown', this.closePickerOnTab, true);
        window.removeEventListener('click', this.closePickerOnOutsideClick, true);
        this.datePicker.close();
    };
    /**
     * @return {?}
     */
    DatePickerComponent.prototype.ngOnDestroy = function () {
        window.removeEventListener('keydown', this.closePickerOnTab, true);
        window.removeEventListener('click', this.closePickerOnOutsideClick, true);
    };
    return DatePickerComponent;
}());
DatePickerComponent.decorators = [
    { type: Component, args: [{
                selector: '',
                template: ''
            },] },
];
/**
 * @nocollapse
 */
DatePickerComponent.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: AngularDatepickerOptions, },
]; };
DatePickerComponent.propDecorators = {
    'class': [{ type: Input },],
    'opened': [{ type: Input },],
    'options': [{ type: Input },],
};

var DatePickerContainerComponent = (function () {
    /**
     * @param {?} elRef
     * @param {?} zone
     */
    function DatePickerContainerComponent(elRef, zone) {
        var _this = this;
        this.elRef = elRef;
        this.zone = zone;
        this.onDateSelected = new EventEmitter();
        this.opened = false;
        this.date = moment();
        this.days = [];
        this.options = {};
        this.watchMediaQuery = function (mql) {
            _this.zone.run(function () {
                _this.mqMatches = _this.mq.matches ? true : false;
            });
        };
    }
    /**
     * @param {?} options
     * @return {?}
     */
    DatePickerContainerComponent.prototype.mergeOptions = function (options) {
        Object.assign(this.options, options);
    };
    /**
     * @param {?=} selectedMonthFlag
     * @return {?}
     */
    DatePickerContainerComponent.prototype.generateCalendar = function (selectedMonthFlag) {
        this.weekDayNames = moment.weekdaysShort();
        if (!this.firstWeekdaySunday) {
            this.weekDayNames.splice(6, 0, this.weekDayNames.splice(0, 1)[0]);
        }
        this.date = selectedMonthFlag && this.inputDate ? this.inputDate.clone() : moment(this.date);
        var /** @type {?} */ month = this.date.month();
        var /** @type {?} */ firstDay = this.date.clone().startOf('month');
        var /** @type {?} */ offset;
        if (this.firstWeekdaySunday) {
            offset = firstDay.day() === 0 ? 7 : firstDay.day();
        }
        else {
            offset = firstDay.day() === 0 ? 6 : firstDay.day() - 1;
        }
        var /** @type {?} */ daysInMonth = this.date.daysInMonth();
        var /** @type {?} */ rows = Math.ceil((offset + daysInMonth) / 7);
        this.weeks = [];
        var /** @type {?} */ currentDate = firstDay.subtract(offset, 'day');
        var /** @type {?} */ selectedDate = this.inputDate;
        for (var /** @type {?} */ i = 1; i <= rows; i += 1) {
            var /** @type {?} */ days = [];
            for (var /** @type {?} */ i_1 = 1; i_1 <= 7; i_1 += 1) {
                var /** @type {?} */ today = (moment().isSame(currentDate, 'day') && moment().isSame(currentDate, 'month')) ? true : false;
                var /** @type {?} */ weekend = (moment(currentDate).day() == 6) || (moment(currentDate).day() == 0);
                var /** @type {?} */ selected = (selectedDate && selectedDate.isSame(currentDate, 'day')) ? true : false;
                days.push({
                    day: currentDate.date(),
                    month: currentDate.month() + 1,
                    year: currentDate.year(),
                    enabled: true,
                    differentMonth: month === currentDate.month(),
                    today: today,
                    weekend: weekend,
                    selected: selected
                });
                currentDate.add(1, 'day');
            }
            this.weeks.push({ rowId: i, days: days });
        }
    };
    /**
     * @return {?}
     */
    DatePickerContainerComponent.prototype.setToday = function () {
        var /** @type {?} */ today = moment();
        this.inputDate = today;
        this.generateCalendar(true);
        this.onDateSelected.emit(this.inputDate);
    };
    /**
     * @param {?} e
     * @param {?} d
     * @return {?}
     */
    DatePickerContainerComponent.prototype.selectDate = function (e, d) {
        e.preventDefault();
        this.inputDate = moment(d.day + "." + d.month + "." + d.year, 'DD.MM.YYYY');
        this.generateCalendar(true);
        this.onDateSelected.emit(this.inputDate);
    };
    /**
     * @return {?}
     */
    DatePickerContainerComponent.prototype.prevMonth = function () {
        this.date = this.date.subtract(1, 'month');
        this.generateCalendar();
    };
    /**
     * @return {?}
     */
    DatePickerContainerComponent.prototype.nextMonth = function () {
        this.date = this.date.add(1, 'month');
        this.generateCalendar();
    };
    /**
     * @return {?}
     */
    DatePickerContainerComponent.prototype.open = function () {
        this.opened = true;
        this.generateCalendar(true);
        if (this.options.modalMediaQuery) {
            this.mq = window.matchMedia("(" + this.options.modalMediaQuery + ")");
            this.mqMatches = this.mq.matches ? true : false;
            this.mq.addListener(this.watchMediaQuery);
        }
    };
    /**
     * @return {?}
     */
    DatePickerContainerComponent.prototype.close = function () {
        this.opened = false;
        if (this.mq) {
            this.mq.removeListener(this.watchMediaQuery);
        }
    };
    /**
     * @return {?}
     */
    DatePickerContainerComponent.prototype.ngOnDestroy = function () {
        if (this.mq) {
            this.mq.removeListener(this.watchMediaQuery);
        }
    };
    return DatePickerContainerComponent;
}());
DatePickerContainerComponent.decorators = [
    { type: Component, args: [{
                // moduleId: module.id,
                selector: 'jb-datepicker-container',
                templateUrl: './datepicker-container.component.html',
                styleUrls: ['./datepicker-container.component.css'],
                animations: [
                    trigger('contentAnimation', [
                        transition('void => *', [
                            style({ opacity: '0' }),
                            animate('200ms ease-in')
                        ]),
                        transition('* => void', [
                            animate('300ms ease-out', style({ opacity: '0' }))
                        ])
                    ])
                ]
            },] },
];
/**
 * @nocollapse
 */
DatePickerContainerComponent.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: NgZone, },
]; };
DatePickerContainerComponent.propDecorators = {
    'onDateSelected': [{ type: Output },],
};

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CALENDAR_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return InputDatePickerComponent; }),
    multi: true
};
var InputDatePickerComponent = (function (_super) {
    __extends(InputDatePickerComponent, _super);
    /**
     * @param {?} elRef
     * @param {?} opts
     */
    function InputDatePickerComponent(elRef, opts) {
        var _this = _super.call(this, elRef, opts) || this;
        _this.elRef = elRef;
        _this.opts = opts;
        return _this;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    InputDatePickerComponent.prototype.viewDateChanged = function (event) {
        if (event && event.length > 0) {
            this.onChangeCallback(event);
        }
        else {
            this.onChangeCallback(null);
        }
    };
    /**
     * @param {?=} event
     * @return {?}
     */
    InputDatePickerComponent.prototype.toggle = function (event) {
        if (event) {
            event.preventDefault();
            event.stopPropagation();
        }
        if (!this.datePicker.opened) {
            if (this.viewDate) {
                this.inputEl.nativeElement.setSelectionRange(0, this.viewDate.length);
            }
            this.open(event);
        }
        else {
            // this.close();
        }
    };
    return InputDatePickerComponent;
}(DatePickerComponent));
InputDatePickerComponent.decorators = [
    { type: Component, args: [{
                // moduleId: module.id,
                selector: 'jb-input-datepicker',
                templateUrl: './input-datepicker.component.html',
                styleUrls: ['./input-datepicker.component.css'],
                providers: [CALENDAR_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
InputDatePickerComponent.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: AngularDatepickerOptions, decorators: [{ type: Optional },] },
]; };
InputDatePickerComponent.propDecorators = {
    'datePicker': [{ type: ViewChild, args: [DatePickerContainerComponent,] },],
    'inputEl': [{ type: ViewChild, args: ['input',] },],
    'options': [{ type: Input },],
};

var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var CALENDAR_VALUE_ACCESSOR$1 = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return TextDatePickerComponent; }),
    multi: true
};
var TextDatePickerComponent = (function (_super) {
    __extends$1(TextDatePickerComponent, _super);
    /**
     * @param {?} elRef
     * @param {?} opts
     */
    function TextDatePickerComponent(elRef, opts) {
        var _this = _super.call(this, elRef, opts) || this;
        _this.elRef = elRef;
        _this.opts = opts;
        return _this;
    }
    return TextDatePickerComponent;
}(DatePickerComponent));
TextDatePickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'jb-text-datepicker',
                templateUrl: './text-datepicker.component.html',
                styleUrls: ['./text-datepicker.component.css'],
                providers: [CALENDAR_VALUE_ACCESSOR$1]
            },] },
];
/**
 * @nocollapse
 */
TextDatePickerComponent.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: AngularDatepickerOptions, decorators: [{ type: Optional },] },
]; };
TextDatePickerComponent.propDecorators = {
    'datePicker': [{ type: ViewChild, args: [DatePickerContainerComponent,] },],
    'options': [{ type: Input },],
};

var AngularDatepickerModule = (function () {
    function AngularDatepickerModule() {
    }
    /**
     * @param {?} config
     * @return {?}
     */
    AngularDatepickerModule.forRoot = function (config) {
        return {
            ngModule: AngularDatepickerModule,
            providers: [
                { provide: AngularDatepickerOptions, useValue: config }
            ]
        };
    };
    return AngularDatepickerModule;
}());
AngularDatepickerModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    ReactiveFormsModule,
                    FormsModule
                ],
                declarations: [
                    DatePickerComponent,
                    DatePickerContainerComponent,
                    InputDatePickerComponent,
                    TextDatePickerComponent
                ],
                providers: [],
                exports: [
                    DatePickerComponent,
                    DatePickerContainerComponent,
                    InputDatePickerComponent,
                    TextDatePickerComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
AngularDatepickerModule.ctorParameters = function () { return []; };

/**
 * Generated bundle index. Do not edit.
 */

export { DatePickerComponent, DatePickerContainerComponent, InputDatePickerComponent, TextDatePickerComponent, AngularDatepickerOptions, AngularDatepickerModule, CALENDAR_VALUE_ACCESSOR as ɵa, CALENDAR_VALUE_ACCESSOR$1 as ɵb };
