import { Component, ElementRef, EventEmitter, Injectable, Input, NgModule, NgZone, Optional, Output, ViewChild, animate, forwardRef, style, transition, trigger } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, NG_VALUE_ACCESSOR, ReactiveFormsModule } from '@angular/forms';
import moment from 'moment';

class AngularDatepickerOptions {
    constructor() {
        this.format = 'DD.MM.YYYY';
        this.viewFormat = 'DD.MM.YYYY';
        this.firstWeekdaySunday = false;
        this.todayButtonEnabled = true;
        this.todayHighlight = true;
        this.weekendHighlight = true;
        this.closeOnSelect = true;
    }
}
AngularDatepickerOptions.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
AngularDatepickerOptions.ctorParameters = () => [];

class DatePickerComponent {
    /**
     * @param {?} elRef
     * @param {?} opts
     */
    constructor(elRef, opts) {
        this.elRef = elRef;
        this.opts = opts;
        this.options = {};
        this.date = moment();
        this.viewDate = null;
        this.days = [];
        this.onTouchedCallback = () => { };
        this.onChangeCallback = () => { };
        this.closePickerOnOutsideClick = (event) => this.close(event);
        this.closePickerOnTab = (event) => this.closeOnTab(event);
    }
    /**
     * @return {?}
     */
    get value() {
        return this.viewDate;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) {
        let /** @type {?} */ date = (value instanceof moment) ? value : moment(value, this.options.format);
        this.viewDate = date.isValid() ? date.format(this.options.viewFormat) : value;
    }
    /**
     * @param {?} date
     * @return {?}
     */
    onDateSelected(date) {
        this.value = date;
        this.onChangeCallback(date.format(this.options.format));
        if (this.options.closeOnSelect) {
            this.close();
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.opts) {
            if (typeof this.options.format === 'undefined') {
                this.options.format = this.opts.format;
            }
            if (typeof this.options.viewFormat === 'undefined') {
                this.options.viewFormat = this.opts.viewFormat;
            }
            if (typeof this.options.modalMediaQuery === 'undefined') {
                this.options.modalMediaQuery = this.opts.modalMediaQuery;
            }
            if (typeof this.options.firstWeekdaySunday === 'undefined') {
                this.options.firstWeekdaySunday = this.opts.firstWeekdaySunday;
            }
            else {
                this.options.firstWeekdaySunday = this.options.firstWeekdaySunday.toString() === 'true';
            }
            if (typeof this.options.todayString === 'undefined') {
                this.options.todayString = this.opts.todayString;
            }
            if (typeof this.options.todayButtonEnabled === 'undefined') {
                this.options.todayButtonEnabled = this.opts.todayButtonEnabled;
            }
            else {
                this.options.todayButtonEnabled = this.options.todayButtonEnabled.toString() === 'true';
            }
            if (typeof this.options.todayHighlight === 'undefined') {
                this.options.todayHighlight = this.opts.todayHighlight;
            }
            else {
                this.options.todayHighlight = this.options.todayHighlight.toString() === 'true';
            }
            if (typeof this.options.weekendHighlight === 'undefined') {
                this.options.weekendHighlight = this.opts.weekendHighlight;
            }
            else {
                this.options.weekendHighlight = this.options.weekendHighlight.toString() === 'true';
            }
            if (typeof this.options.closeOnSelect === 'undefined') {
                this.options.closeOnSelect = this.opts.closeOnSelect;
            }
            else {
                this.options.closeOnSelect = this.options.closeOnSelect.toString() === 'true';
            }
        }
        this.datePicker.mergeOptions(this.options);
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.opened) {
            this.datePicker.open();
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.viewDate = value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    /**
     * @return {?}
     */
    toggle() {
        if (!this.datePicker.opened) {
            this.open();
        }
        else {
            // this.close();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    closeOnTab(event) {
        if (event.keyCode === 9) {
            if (this.viewDate) {
                this.value = moment(this.viewDate, this.options.viewFormat);
            }
            this.onTouchedCallback();
            this.close();
        }
    }
    /**
     * @param {?=} event
     * @return {?}
     */
    open(event) {
        if (event) {
            event.preventDefault();
        }
        let /** @type {?} */ date = moment(this.value, this.options.viewFormat);
        this.datePicker.inputDate = date.isValid() ? date : null;
        this.datePicker.open();
        window.addEventListener('click', this.closePickerOnOutsideClick, true);
        window.addEventListener('keydown', this.closePickerOnTab, true);
    }
    /**
     * @param {?=} event
     * @return {?}
     */
    close(event) {
        if (event && this.elRef.nativeElement.contains(event.target)) {
            return;
        }
        this.onTouchedCallback();
        window.removeEventListener('keydown', this.closePickerOnTab, true);
        window.removeEventListener('click', this.closePickerOnOutsideClick, true);
        this.datePicker.close();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        window.removeEventListener('keydown', this.closePickerOnTab, true);
        window.removeEventListener('click', this.closePickerOnOutsideClick, true);
    }
}
DatePickerComponent.decorators = [
    { type: Component, args: [{
                selector: '',
                template: ''
            },] },
];
/**
 * @nocollapse
 */
DatePickerComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: AngularDatepickerOptions, },
];
DatePickerComponent.propDecorators = {
    'class': [{ type: Input },],
    'opened': [{ type: Input },],
    'options': [{ type: Input },],
};

class DatePickerContainerComponent {
    /**
     * @param {?} elRef
     * @param {?} zone
     */
    constructor(elRef, zone) {
        this.elRef = elRef;
        this.zone = zone;
        this.onDateSelected = new EventEmitter();
        this.opened = false;
        this.date = moment();
        this.days = [];
        this.options = {};
        this.watchMediaQuery = (mql) => {
            this.zone.run(() => {
                this.mqMatches = this.mq.matches ? true : false;
            });
        };
    }
    /**
     * @param {?} options
     * @return {?}
     */
    mergeOptions(options) {
        Object.assign(this.options, options);
    }
    /**
     * @param {?=} selectedMonthFlag
     * @return {?}
     */
    generateCalendar(selectedMonthFlag) {
        this.weekDayNames = moment.weekdaysShort();
        if (!this.firstWeekdaySunday) {
            this.weekDayNames.splice(6, 0, this.weekDayNames.splice(0, 1)[0]);
        }
        this.date = selectedMonthFlag && this.inputDate ? this.inputDate.clone() : moment(this.date);
        let /** @type {?} */ month = this.date.month();
        let /** @type {?} */ firstDay = this.date.clone().startOf('month');
        let /** @type {?} */ offset;
        if (this.firstWeekdaySunday) {
            offset = firstDay.day() === 0 ? 7 : firstDay.day();
        }
        else {
            offset = firstDay.day() === 0 ? 6 : firstDay.day() - 1;
        }
        let /** @type {?} */ daysInMonth = this.date.daysInMonth();
        let /** @type {?} */ rows = Math.ceil((offset + daysInMonth) / 7);
        this.weeks = [];
        let /** @type {?} */ currentDate = firstDay.subtract(offset, 'day');
        let /** @type {?} */ selectedDate = this.inputDate;
        for (let /** @type {?} */ i = 1; i <= rows; i += 1) {
            let /** @type {?} */ days = [];
            for (let /** @type {?} */ i = 1; i <= 7; i += 1) {
                let /** @type {?} */ today = (moment().isSame(currentDate, 'day') && moment().isSame(currentDate, 'month')) ? true : false;
                let /** @type {?} */ weekend = (moment(currentDate).day() == 6) || (moment(currentDate).day() == 0);
                let /** @type {?} */ selected = (selectedDate && selectedDate.isSame(currentDate, 'day')) ? true : false;
                days.push({
                    day: currentDate.date(),
                    month: currentDate.month() + 1,
                    year: currentDate.year(),
                    enabled: true,
                    differentMonth: month === currentDate.month(),
                    today: today,
                    weekend: weekend,
                    selected: selected
                });
                currentDate.add(1, 'day');
            }
            this.weeks.push({ rowId: i, days: days });
        }
    }
    /**
     * @return {?}
     */
    setToday() {
        let /** @type {?} */ today = moment();
        this.inputDate = today;
        this.generateCalendar(true);
        this.onDateSelected.emit(this.inputDate);
    }
    /**
     * @param {?} e
     * @param {?} d
     * @return {?}
     */
    selectDate(e, d) {
        e.preventDefault();
        this.inputDate = moment(`${d.day}.${d.month}.${d.year}`, 'DD.MM.YYYY');
        this.generateCalendar(true);
        this.onDateSelected.emit(this.inputDate);
    }
    /**
     * @return {?}
     */
    prevMonth() {
        this.date = this.date.subtract(1, 'month');
        this.generateCalendar();
    }
    /**
     * @return {?}
     */
    nextMonth() {
        this.date = this.date.add(1, 'month');
        this.generateCalendar();
    }
    /**
     * @return {?}
     */
    open() {
        this.opened = true;
        this.generateCalendar(true);
        if (this.options.modalMediaQuery) {
            this.mq = window.matchMedia(`(${this.options.modalMediaQuery})`);
            this.mqMatches = this.mq.matches ? true : false;
            this.mq.addListener(this.watchMediaQuery);
        }
    }
    /**
     * @return {?}
     */
    close() {
        this.opened = false;
        if (this.mq) {
            this.mq.removeListener(this.watchMediaQuery);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.mq) {
            this.mq.removeListener(this.watchMediaQuery);
        }
    }
}
DatePickerContainerComponent.decorators = [
    { type: Component, args: [{
                // moduleId: module.id,
                selector: 'jb-datepicker-container',
                templateUrl: './datepicker-container.component.html',
                styleUrls: ['./datepicker-container.component.css'],
                animations: [
                    trigger('contentAnimation', [
                        transition('void => *', [
                            style({ opacity: '0' }),
                            animate('200ms ease-in')
                        ]),
                        transition('* => void', [
                            animate('300ms ease-out', style({ opacity: '0' }))
                        ])
                    ])
                ]
            },] },
];
/**
 * @nocollapse
 */
DatePickerContainerComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: NgZone, },
];
DatePickerContainerComponent.propDecorators = {
    'onDateSelected': [{ type: Output },],
};

const CALENDAR_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => InputDatePickerComponent),
    multi: true
};
class InputDatePickerComponent extends DatePickerComponent {
    /**
     * @param {?} elRef
     * @param {?} opts
     */
    constructor(elRef, opts) {
        super(elRef, opts);
        this.elRef = elRef;
        this.opts = opts;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    viewDateChanged(event) {
        if (event && event.length > 0) {
            this.onChangeCallback(event);
        }
        else {
            this.onChangeCallback(null);
        }
    }
    /**
     * @param {?=} event
     * @return {?}
     */
    toggle(event) {
        if (event) {
            event.preventDefault();
            event.stopPropagation();
        }
        if (!this.datePicker.opened) {
            if (this.viewDate) {
                this.inputEl.nativeElement.setSelectionRange(0, this.viewDate.length);
            }
            this.open(event);
        }
        else {
            // this.close();
        }
    }
}
InputDatePickerComponent.decorators = [
    { type: Component, args: [{
                // moduleId: module.id,
                selector: 'jb-input-datepicker',
                templateUrl: './input-datepicker.component.html',
                styleUrls: ['./input-datepicker.component.css'],
                providers: [CALENDAR_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
InputDatePickerComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: AngularDatepickerOptions, decorators: [{ type: Optional },] },
];
InputDatePickerComponent.propDecorators = {
    'datePicker': [{ type: ViewChild, args: [DatePickerContainerComponent,] },],
    'inputEl': [{ type: ViewChild, args: ['input',] },],
    'options': [{ type: Input },],
};

const CALENDAR_VALUE_ACCESSOR$1 = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => TextDatePickerComponent),
    multi: true
};
class TextDatePickerComponent extends DatePickerComponent {
    /**
     * @param {?} elRef
     * @param {?} opts
     */
    constructor(elRef, opts) {
        super(elRef, opts);
        this.elRef = elRef;
        this.opts = opts;
    }
}
TextDatePickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'jb-text-datepicker',
                templateUrl: './text-datepicker.component.html',
                styleUrls: ['./text-datepicker.component.css'],
                providers: [CALENDAR_VALUE_ACCESSOR$1]
            },] },
];
/**
 * @nocollapse
 */
TextDatePickerComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: AngularDatepickerOptions, decorators: [{ type: Optional },] },
];
TextDatePickerComponent.propDecorators = {
    'datePicker': [{ type: ViewChild, args: [DatePickerContainerComponent,] },],
    'options': [{ type: Input },],
};

class AngularDatepickerModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: AngularDatepickerModule,
            providers: [
                { provide: AngularDatepickerOptions, useValue: config }
            ]
        };
    }
}
AngularDatepickerModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    ReactiveFormsModule,
                    FormsModule
                ],
                declarations: [
                    DatePickerComponent,
                    DatePickerContainerComponent,
                    InputDatePickerComponent,
                    TextDatePickerComponent
                ],
                providers: [],
                exports: [
                    DatePickerComponent,
                    DatePickerContainerComponent,
                    InputDatePickerComponent,
                    TextDatePickerComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
AngularDatepickerModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { DatePickerComponent, DatePickerContainerComponent, InputDatePickerComponent, TextDatePickerComponent, AngularDatepickerOptions, AngularDatepickerModule, CALENDAR_VALUE_ACCESSOR as ɵa, CALENDAR_VALUE_ACCESSOR$1 as ɵb };
